@@ -77,6 +77,24 @@ void TestExpressionParser::newSelf()
     QCOMPARE(StructureType::Ptr::staticCast(res.type())->qualifiedIdentifier(), QualifiedIdentifier("a"));
 }
 
+void TestExpressionParser::newStatic()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? class A { function self() {  } }");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    ExpressionParser p(true);
+    ExpressionEvaluationResult res = p.evaluateType( QByteArray("new static()"),
+                                        DUContextPointer(top->childContexts().first()->childContexts().last()),
+                                        CursorInRevision(0, 30));
+    QVERIFY(res.type().cast<StructureType>());
+    QCOMPARE(res.type().cast<StructureType>()->qualifiedIdentifier(), QualifiedIdentifier("a"));
+}
+
 void TestExpressionParser::memberVariable()
 {
     //                 0         1         2         3         4         5         6         7
@@ -112,6 +130,92 @@ void TestExpressionParser::memberFunction()
     QCOMPARE(res.allDeclarations().size(), 1);
     QCOMPARE(res.allDeclarations().first().data(), top->childContexts().first()->localDeclarations().first());
 }
+
+void TestExpressionParser::newTrait()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? trait A { function foo() {} }");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+
+    QVERIFY(top->problems().isEmpty());
+
+    QCOMPARE(top->childContexts().size(), 1);
+    QVERIFY(top->childContexts().at(0)->type() == DUContext::Class);
+}
+
+void TestExpressionParser::newTraitWithAbstractMethod()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? trait A { public abstract function foo(); }");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+
+    QVERIFY(top->problems().isEmpty());
+
+    QCOMPARE(top->childContexts().size(), 1);
+    QVERIFY(top->childContexts().at(0)->type() == DUContext::Class);
+}
+
+void TestExpressionParser::invalidTrait_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("constant") << "<? trait A { const FOO = ''; }\n";
+}
+
+void TestExpressionParser::invalidTrait()
+{
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(!top->problems().isEmpty());
+}
+
+void TestExpressionParser::invalidTraitUse_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("staticModifier") << "<? trait A { public function foo(){} } class Foo { use A { A::foo as public static bla; } }\n";
+
+    QTest::newRow("finalModifier") << "<? trait A { public function foo(){} } class Foo { use A { A::foo as public final bla; } }\n";
+
+    QTest::newRow("traitMethodCollision") << "<? trait A { public function foo(){} } trait B { public function foo(){} } class Foo { use A,B; }\n";
+
+    QTest::newRow("propertyCollision") << "<? trait A { public $arg; } class Foo { use A; public $arg; }\n";
+}
+
+void TestExpressionParser::invalidTraitUse()
+{
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(!top->problems().isEmpty());
+}
+
+void TestExpressionParser::namespaceUseNameConflict()
+{
+    QByteArray alias("<?php namespace Bar { class Foo {} use Foo; }\n");
+
+    TopDUContext* top = parse(alias, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(!top->problems().isEmpty());
+}
+
 void TestExpressionParser::globalFunction()
 {
     //                 0         1         2         3         4         5         6         7
@@ -368,18 +472,33 @@ void TestExpressionParser::findArg()
     QCOMPARE(StructureType::Ptr::staticCast(type->baseType())->declaration(top), top->localDeclarations().first());
 }
 
+void TestExpressionParser::array_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? $a = array(1,2,3);\n";
+
+    QTest::newRow("shortSyntax") << "<? $a = [1,2,3];\n";
+
+    QTest::newRow("staticNormalSyntax") << "<? static $a = array(1,2,3);\n";
+
+    QTest::newRow("staticShortSyntax") << "<? static $a = [1,2,3];\n";
+}
+
 void TestExpressionParser::array()
 {
     // see bug https://bugs.kde.org/show_bug.cgi?id=237110
 
     //                 0         1         2         3         4         5         6         7
     //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
-    QByteArray method("<? $a = array(\"foo\");");
+    QFETCH(QString, code);
 
-    TopDUContext* top = parse(method, DumpNone);
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
     DUChainReleaser releaseTop(top);
     DUChainWriteLocker lock;
 
+    QVERIFY(top->problems().isEmpty());
+
     ExpressionParser p(true);
     QCOMPARE(top->localDeclarations().first()->abstractType().cast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeArray));
 
@@ -391,6 +510,121 @@ void TestExpressionParser::array()
     QCOMPARE(res.type().cast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeMixed));
 }
 
+void TestExpressionParser::arrayFunctionDereferencing_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("globalFunction") << "<? function foo() { return [1, 2]; }\n"
+                                       "$a = foo()[0];\n";
+
+    QTest::newRow("classMethod") << "<? class foo{ function bar() { return [1, 2]; } }\n"
+                                       "$obj = new foo(); $a = $obj->bar()[0];\n";
+
+    QTest::newRow("staticClassMethod") << "<? class foo{ static function bar() { return [1, 2]; } }\n"
+                                       "$a = foo::bar()[0];\n";
+}
+
+void TestExpressionParser::arrayFunctionDereferencing()
+{
+    // see bug https://bugs.kde.org/show_bug.cgi?id=305779
+    QFETCH(QString, code);
+    /*
+    QByteArray method("<? function foo() { return [1, 2]; }\n"
+                      "class obj { static function bar() { return [1, 2]; } }\n"
+                      "$myObj = new obj();\n"
+                      // actual test stuff:
+                      "$a = foo()[0];\n"
+                      "$b = $myObj->bar()[0];\n"
+                      "$c = obj::bar()[0];\n");*/
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(top->problems().isEmpty());
+
+    Declaration* decl = top->localDeclarations().last();
+    IntegralType::Ptr type = decl->abstractType().cast<IntegralType>();
+    QVERIFY(type);
+    QEXPECT_FAIL("", "we'd need advanced array support to know that [0] returns a int...", Continue);
+    QCOMPARE(type->dataType(), static_cast<uint>(IntegralType::TypeInt));
+    // fallback
+    QCOMPARE(type->dataType(), static_cast<uint>(IntegralType::TypeMixed));
+}
+
+void TestExpressionParser::arrayLiteralDereferencing_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? $a = array(1,2,3)[1];\n";
+
+    QTest::newRow("shortSyntax") << "<? $a = [1,2,3][1];\n";
+}
+
+void TestExpressionParser::arrayLiteralDereferencing()
+{
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(top->problems().isEmpty());
+
+    Declaration* decl = top->localDeclarations().last();
+    IntegralType::Ptr type = decl->abstractType().cast<IntegralType>();
+    QVERIFY(type);
+    QEXPECT_FAIL("", "we'd need advanced array support to know that [0] returns a int...", Continue);
+    QCOMPARE(type->dataType(), static_cast<uint>(IntegralType::TypeInt));
+    // fallback
+    QCOMPARE(type->dataType(), static_cast<uint>(IntegralType::TypeMixed));
+}
+
+void TestExpressionParser::stringAsArray_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("constantEncapsedString") << "<? $a = 'string'[1];\n";
+
+    QTest::newRow("dynamicString") << "<? $string = 'Hello';\n"
+                                      "$a = \"$string World\"[1];\n";
+}
+
+void TestExpressionParser::stringAsArray()
+{
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(top->problems().isEmpty());
+
+    Declaration* decl = top->localDeclarations().last();
+    IntegralType::Ptr type = decl->abstractType().cast<IntegralType>();
+    QVERIFY(type);
+    QCOMPARE(type->dataType(), static_cast<uint>(IntegralType::TypeString));
+}
+
+void TestExpressionParser::classMemberOnInstantiation()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? class A{ function foo(){ return 'a'; } } $a = (new A())->foo();");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(top->problems().isEmpty());
+
+    ExpressionParser p(true);
+
+    ExpressionEvaluationResult res = p.evaluateType(QByteArray("$a"), DUContextPointer(top), CursorInRevision(1, 0));
+    QVERIFY(res.type());
+    QCOMPARE(IntegralType::Ptr::staticCast(res.type())->dataType(), static_cast<uint>(IntegralType::TypeString));
+}
+
 }
 
 #include "expressionparser.moc"
