@@ -26,27 +26,13 @@
 #include "phpparsejob.h"
 #include "../declarations/classdeclaration.h"
 #include "../declarations/variabledeclaration.h"
+#include "../declarations/traitmethodaliasdeclaration.h"
+#include "../declarations/traitmemberaliasdeclaration.h"
 
 using namespace KDevelop;
 
 QTEST_MAIN(Php::TestUses)
 
-namespace QTest {
-
-template<>
-char* toString(const KDevelop::CursorInRevision& c)
-{
-    return qstrdup(qPrintable(QString("(%1, %2)").arg(c.line).arg(c.column)));
-}
-
-template<>
-char* toString(const KDevelop::RangeInRevision& r)
-{
-    return qstrdup(qPrintable(QString("[%1, %2]").arg(toString(r.start)).arg(toString(r.end))));
-}
-
-}
-
 namespace Php
 {
 
@@ -155,6 +141,47 @@ void TestUses::varInString()
     compareUses(top->localDeclarations().at(0), ranges);
 }
 
+void TestUses::variableInNamespace()
+{
+
+    //                        0         1         2         3         4         5         6         7
+    //                        01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<?php\nclass A { public $foo; } namespace Foo { $a = new A(); $a; $a->foo; foo($a); };");
+    TopDUContext* top = parse(method, DumpAll);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+    QList<RangeInRevision> ranges;
+    ranges << RangeInRevision(1, 55, 1, 57) << RangeInRevision(1, 59, 1, 61) << RangeInRevision(1, 72, 1, 74);
+    compareUses(top->localDeclarations().at(2), ranges);
+}
+
+void TestUses::globalVariableInNamespace()
+{
+
+    //                        0         1         2         3         4         5         6         7
+    //                        01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<?php\nclass A { public $foo; } $a = new A(); namespace Foo { $a; $a->foo; foo($a); };");
+    TopDUContext* top = parse(method, DumpAll);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+    QList<RangeInRevision> ranges;
+    ranges << RangeInRevision(1, 55, 1, 57) << RangeInRevision(1, 59, 1, 61) << RangeInRevision(1, 72, 1, 74);
+    compareUses(top->localDeclarations().at(1), ranges);
+}
+
+void TestUses::variableInOtherNamespace()
+{
+    //                        0         1         2         3         4         5         6         7
+    //                        01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<?php\nclass A { public $foo; } namespace Foo { $a = new A(); } namespace Bar { $a; $a->foo; foo($a); };");
+    TopDUContext* top = parse(method, DumpAll);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+    QList<RangeInRevision> ranges;
+    ranges << RangeInRevision(1, 73, 1, 75) << RangeInRevision(1, 77, 1, 79) << RangeInRevision(1, 90, 1, 92);
+    compareUses(top->localDeclarations().at(2), ranges);
+}
+
 void TestUses::memberVarInString()
 {
 
@@ -872,6 +899,7 @@ void TestUses::namespaces()
     TopDUContext* top = parse("<?php\n"
                               "namespace Foo\\Bar {\n"
                               "const MyConst = 1;\n"
+                              "$z = MyConst;\n"
                               "function MyFunc(){}\n"
                               "class MyClass{ const ClassConst = 2; }\n"
                               "interface MyInterface{}\n"
@@ -895,48 +923,49 @@ void TestUses::namespaces()
     dec = top->findDeclarations(QualifiedIdentifier("foo")).last();
     QCOMPARE(dec->kind(), Declaration::Namespace);
     compareUses(dec, QList<RangeInRevision>()
-                                          << RangeInRevision(8, 1, 8, 4)
                                           << RangeInRevision(9, 1, 9, 4)
                                           << RangeInRevision(10, 1, 10, 4)
-                                          << RangeInRevision(11, 5, 11, 8)
-                                          << RangeInRevision(12, 15, 12, 18)
-                                          << RangeInRevision(13, 17, 13, 20)
-                                          << RangeInRevision(13, 45, 13, 48));
+                                          << RangeInRevision(11, 1, 11, 4)
+                                          << RangeInRevision(12, 5, 12, 8)
+                                          << RangeInRevision(13, 15, 13, 18)
+                                          << RangeInRevision(14, 17, 14, 20)
+                                          << RangeInRevision(14, 45, 14, 48));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar")).first();
     QCOMPARE(dec->kind(), Declaration::Namespace);
     QVERIFY(dec->internalContext());
-    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(8, 5, 8, 8)
-                                          << RangeInRevision(9, 5, 9, 8)
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(9, 5, 9, 8)
                                           << RangeInRevision(10, 5, 10, 8)
-                                          << RangeInRevision(11, 9, 11, 12)
-                                          << RangeInRevision(12, 19, 12, 22)
-                                          << RangeInRevision(13, 21, 13, 24)
-                                          << RangeInRevision(13, 49, 13, 52));
+                                          << RangeInRevision(11, 5, 11, 8)
+                                          << RangeInRevision(12, 9, 12, 12)
+                                          << RangeInRevision(13, 19, 13, 22)
+                                          << RangeInRevision(14, 21, 14, 24)
+                                          << RangeInRevision(14, 49, 14, 52));
     QCOMPARE(dec->internalContext()->localDeclarations().size(), 4);
     foreach(Declaration* d, dec->internalContext()->localDeclarations()) {
         kDebug() << d->toString() << d->qualifiedIdentifier();
     }
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::MyConst")).first();
-    compareUses(dec, RangeInRevision(8, 9, 8, 16));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(3, 5, 3, 12)
+                                          << RangeInRevision(9, 9, 9, 16));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myclass")).first();
     QVERIFY(dynamic_cast<ClassDeclaration*>(dec));
-    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(9, 9, 9, 16)
-                                          << RangeInRevision(11, 13, 11, 20)
-                                          << RangeInRevision(12, 23, 12, 30)
-                                          << RangeInRevision(13, 25, 13, 32)
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(10, 9, 10, 16)
+                                          << RangeInRevision(12, 13, 12, 20)
+                                          << RangeInRevision(13, 23, 13, 30)
+                                          << RangeInRevision(14, 25, 14, 32)
                );
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myinterface")).first();
     QVERIFY(dynamic_cast<ClassDeclaration*>(dec));
-    compareUses(dec, RangeInRevision(13, 53, 13, 64) );
+    compareUses(dec, RangeInRevision(14, 53, 14, 64) );
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myclass::ClassConst")).first();
-    compareUses(dec, RangeInRevision(9, 18, 9, 28));
+    compareUses(dec, RangeInRevision(10, 18, 10, 28));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myfunc")).first();
-    compareUses(dec, RangeInRevision(10, 9, 10, 15));
+    compareUses(dec, RangeInRevision(11, 9, 11, 15));
 }
 
 void TestUses::useNamespace()
@@ -946,10 +975,13 @@ void TestUses::useNamespace()
     TopDUContext* top = parse("<?php\n"
                               "namespace Foo\\Bar {class A{} function B(){} const C = 1;}\n"
                               "namespace VeryLong {class A{} function B(){} const C = 1;}\n"
+                              "namespace Baz {class A{} const C = 1;}\n"
                               "namespace {\n"
                               "use Foo\\Bar, VeryLong as Short;\n"
+                              "use Baz\\A as Bazaar;\n"
                               "new Bar\\A; Bar\\B(); Bar\\C;\n"
                               "new Short\\A; Short\\B(); Short\\C;\n"
+                              "new Bazaar;\n"
                               "}\n", DumpNone);
     QVERIFY(top);
     DUChainReleaser releaseTop(top);
@@ -958,45 +990,49 @@ void TestUses::useNamespace()
     Declaration* dec;
     dec = top->findDeclarations(QualifiedIdentifier("foo")).first();
     QCOMPARE(dec->kind(), Declaration::Namespace);
-    compareUses(dec, RangeInRevision(4, 4, 4, 7));
+    compareUses(dec, RangeInRevision(5, 4, 5, 7));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar")).first();
     QCOMPARE(dec->kind(), Declaration::Namespace);
-    compareUses(dec, RangeInRevision(4, 8, 4, 11));
+    compareUses(dec, RangeInRevision(5, 8, 5, 11));
 
     dec = top->findDeclarations(QualifiedIdentifier("verylong")).first();
     QCOMPARE(dec->kind(), Declaration::Namespace);
-    compareUses(dec, RangeInRevision(4, 13, 4, 21));
+    compareUses(dec, RangeInRevision(5, 13, 5, 21));
 
     dec = top->findDeclarations(QualifiedIdentifier("bar")).first();
     QCOMPARE(dec->kind(), Declaration::NamespaceAlias);
-    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(5, 4, 5, 7)
-                                          << RangeInRevision(5, 11, 5, 14)
-                                          << RangeInRevision(5, 20, 5, 23) );
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(7, 4, 7, 7)
+                                          << RangeInRevision(7, 11, 7, 14)
+                                          << RangeInRevision(7, 20, 7, 23) );
 
     dec = top->findDeclarations(QualifiedIdentifier("short")).first();
     QCOMPARE(dec->kind(), Declaration::NamespaceAlias);
-    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(6, 4, 6, 9)
-                                          << RangeInRevision(6, 13, 6, 18)
-                                          << RangeInRevision(6, 24, 6, 29) );
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(8, 4, 8, 9)
+                                          << RangeInRevision(8, 13, 8, 18)
+                                          << RangeInRevision(8, 24, 8, 29) );
+
+    dec = top->findDeclarations(QualifiedIdentifier("baz::a")).first();
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(6, 8, 6, 9)
+                                          << RangeInRevision(9, 4, 9, 10));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::a")).first();
-    compareUses(dec, RangeInRevision(5, 8, 5, 9));
+    compareUses(dec, RangeInRevision(7, 8, 7, 9));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::b")).first();
-    compareUses(dec, RangeInRevision(5, 15, 5, 16));
+    compareUses(dec, RangeInRevision(7, 15, 7, 16));
 
     dec = top->findDeclarations(QualifiedIdentifier("foo::bar::C")).first();
-    compareUses(dec, RangeInRevision(5, 24, 5, 25));
+    compareUses(dec, RangeInRevision(7, 24, 7, 25));
 
     dec = top->findDeclarations(QualifiedIdentifier("verylong::a")).first();
-    compareUses(dec, RangeInRevision(6, 10, 6, 11));
+    compareUses(dec, RangeInRevision(8, 10, 8, 11));
 
     dec = top->findDeclarations(QualifiedIdentifier("verylong::b")).first();
-    compareUses(dec, RangeInRevision(6, 19, 6, 20));
+    compareUses(dec, RangeInRevision(8, 19, 8, 20));
 
     dec = top->findDeclarations(QualifiedIdentifier("verylong::C")).first();
-    compareUses(dec, RangeInRevision(6, 30, 6, 31));
+    compareUses(dec, RangeInRevision(8, 30, 8, 31));
 }
 
 void TestUses::lateStatic()
@@ -1053,6 +1089,125 @@ void TestUses::instanceof()
 
 }
 
+void TestUses::classNameString()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? class Foo { } echo 'Foo';");
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+
+
+    Declaration* foo = top->localDeclarations().at(0);
+    QCOMPARE(foo->identifier().toString(), QString("foo"));
+    compareUses(foo, RangeInRevision(0, 22, 0, 27));
+}
+
+void TestUses::useTrait()
+{
+    //                         0         1         2         3         4         5         6         7
+    //                         01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    TopDUContext* top = parse("<?php\n"
+                              "trait A { public function one(){} public function two(){} }\n"
+                              "trait B { public function one(){} private function three(){} }\n"
+                              "trait C { public $baz; public function one(){} public static function five(){} }\n"
+                              "class Foo { use A;}\n"
+                              "class Bar { use A,B { A::one insteadof B; B::three as public four; } }\n"
+                              "class Baz { use A,B,C { C::one insteadof A,B; C::five as six; } }\n"
+                              "$a = new Foo(); $a->one(); $a->two();\n"
+                              "$b = new Bar(); $b->one(); $b->two(); $b->four();\n"
+                              "$c = new Baz(); $c->one(); $c->two(); $c->baz; $c::six();\n", DumpAll);
+    QVERIFY(top);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    Declaration* dec;
+    QVector<Declaration*> topDecs = top->localDeclarations();
+    TraitMethodAliasDeclaration* method;
+    TraitMemberAliasDeclaration* member;
+
+    QCOMPARE(topDecs.size(), 9);
+
+    dec = top->findDeclarations(QualifiedIdentifier("a")).first();
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(4, 16, 4, 17)
+                                          << RangeInRevision(5, 16, 5, 17)
+                                          << RangeInRevision(5, 22, 5, 23)
+                                          << RangeInRevision(6, 16, 6, 17)
+                                          << RangeInRevision(6, 41, 6, 42) );
+
+    dec = top->findDeclarations(QualifiedIdentifier("b")).first();
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(5, 18, 5, 19)
+                                          << RangeInRevision(5, 39, 5, 40)
+                                          << RangeInRevision(5, 42, 5, 43)
+                                          << RangeInRevision(6, 18, 6, 19)
+                                          << RangeInRevision(6, 43, 6, 44) );
+
+    dec = top->findDeclarations(QualifiedIdentifier("c")).first();
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(6, 20, 6, 21)
+                                          << RangeInRevision(6, 24, 6, 25)
+                                          << RangeInRevision(6, 46, 6, 47) );
+
+    dec = topDecs[3]->internalContext()->findLocalDeclarations(Identifier("one")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("a"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(7, 20, 7, 23) );
+
+    dec = topDecs[3]->internalContext()->findLocalDeclarations(Identifier("two")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("a"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(7, 31, 7, 34) );
+
+    dec = topDecs[4]->internalContext()->findLocalDeclarations(Identifier("one")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("a"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(8, 20, 8, 23) );
+
+    dec = topDecs[4]->internalContext()->findLocalDeclarations(Identifier("two")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("a"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(8, 31, 8, 34) );
+
+    dec = topDecs[4]->internalContext()->findLocalDeclarations(Identifier("four")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("b"));
+    QCOMPARE(method->aliasedDeclaration().data()->identifier(), Identifier("three"));
+    QCOMPARE(method->accessPolicy(), Declaration::AccessPolicy::Public);
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(8, 42, 8, 46) );
+
+    dec = topDecs[5]->internalContext()->findLocalDeclarations(Identifier("one")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("c"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(9, 20, 9, 23) );
+
+    dec = topDecs[5]->internalContext()->findLocalDeclarations(Identifier("two")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("a"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(9, 31, 9, 34) );
+
+    dec = topDecs[5]->internalContext()->findLocalDeclarations(Identifier("baz")).first();
+    member = dynamic_cast<TraitMemberAliasDeclaration*>(dec);
+    QVERIFY(member);
+    QCOMPARE(member->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("c"));
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(9, 42, 9, 45) );
+
+    dec = topDecs[5]->internalContext()->findLocalDeclarations(Identifier("six")).first();
+    method = dynamic_cast<TraitMethodAliasDeclaration*>(dec);
+    QVERIFY(method);
+    QCOMPARE(method->aliasedDeclaration().data()->context()->owner()->identifier(), Identifier("c"));
+    QCOMPARE(method->aliasedDeclaration().data()->identifier(), Identifier("five"));
+    QVERIFY(method->isStatic());
+    compareUses(dec, QList<RangeInRevision>() << RangeInRevision(9, 51, 9, 54) );
+}
+
+
 }
 
 #include "uses.moc"
