@@ -37,6 +37,9 @@ private slots:
     void memberVariable();
     void variable();
     void varInString();
+    void variableInNamespace();
+    void globalVariableInNamespace();
+    void variableInOtherNamespace();
     void memberVarInString();
     void memberFunctionInString();
     void variableTypeChange();
@@ -78,6 +81,9 @@ private slots:
     void useNamespace();
     void lateStatic();
     void closures();
+    void instanceof();
+    void classNameString();
+    void useTrait();
 };
 
 }
