@@ -1,10 +1,10 @@
 /*
-    <one line to give the program's name and a brief idea of what it does.>
-    Copyright (C) <year>  <name of author>
+    This file is part of KDevelop PHP
+    Copyright (C) 2010 Milian Wolff
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -25,10 +25,13 @@
 using namespace KDevelop;
 using namespace Php;
 
+//BEGIN HighlightingInstance
+
 class HighlightingInstance : public CodeHighlightingInstance {
 public:
     HighlightingInstance(const CodeHighlighting* highlighting);
     virtual Types typeForDeclaration(KDevelop::Declaration* decl, KDevelop::DUContext* context) const;
+    virtual bool useRainbowColor( Declaration* dec ) const;
 };
 
 HighlightingInstance::HighlightingInstance(const CodeHighlighting* highlighting)
@@ -48,6 +51,13 @@ HighlightingEnumContainer::Types HighlightingInstance::typeForDeclaration(Declar
     }
 }
 
+bool HighlightingInstance::useRainbowColor( Declaration* dec ) const
+{
+    return dynamic_cast<VariableDeclaration*>(dec);
+}
+
+//BEGIN Highlighting
+
 Highlighting::Highlighting(QObject* parent)
     : CodeHighlighting(parent)
 {
